#!/usr/bin/env php
<?php

$rc = 0;

$basedir   = strval($_ENV['MYENV_BASE']);

if ( $basedir == '' ) {
  $rc = 30;
  echo "ERROR: MyEnv is not installed yet. Please install MyEnv first with the command installMyEnv\n";
  exit($rc);
}

require("$basedir/lib/myEnv.inc");
$lConfFile = $basedir . '/' . 'etc/' . 'myenv.conf';
debug("ConfFile: $lConfFile");

if ( ! file_exists($lConfFile) ) {
  $rc = 31;
  error("MyEnv configuration file $lConfFile does not exist.");
  exit($rc);
}

$aConfiguration = parse_ini_file($lConfFile, true);

$lProductDir = $_ENV['HOME'] . '/' . 'product';
$lTmpDir     = $_ENV['HOME'] . '/' . 'tmp';

if ( ! is_dir($lProductDir) ) {
  echo "\n";
  echo "INFO: Product directory $lProductDir does not exist yet.\n";
  echo "INFO: Creating it now.\n";
  mkdir($lProductDir, 0755, true);
}

if ( ! is_dir($lTmpDir) ) {
  echo "\n";
  echo "INFO: Tmp directory $lTmpDir does not exist yet.\n";
  echo "INFO: Creating it now.\n";
  mkdir($lTmpDir, 0755, true);
}

// Download URL

echo "\n";
do {
  $url = readline("Please enter the URL for the binary-tarball (Linux non RPM packages) to download: ");
} while ( $url == '');

echo "\n";
$lVersion = extractVersion($url);
if ( $lVersion == '' ) {
  echo "Version cannot be determined.\n";
}
else {
  echo "The version is: $lVersion\n";
}

// todo: compare download with platform

// todo: check if wget or curl is available

// download
echo "\n";
$command = "wget --directory-prefix=$lTmpDir $url 2>&1";
$stdout = exec($command, &$output, &$ret);

if ( $ret != 0 ) {
  $rc = 33;
  error("Download failed. Please try again or manually.");
  exit($rc);
}

// get binary path from stdout!
// Saving to: `/home/myenv/tmp/mysql-5.1.41-linux-i686-glibc23.tar.gz'

if ( preg_match("/$lTmpDir\/(mysql.*.tar.gz)/", $stdout, &$matches) ) {
  $lTarBall = $matches[1];
  echo "Tar-ball file is: $lTarBall\n";
}
else {
  $rc = 34;
  error("Cannot extract filename from output. Please try manually.\n");
  exit($rc);
}

// todo: check checksum

// extract binary
$command = "tar xf $lTmpDir . '/' . $lTarBall --directory $lProductDir";
echo $command . "\n";
$stdout = exec($command, &$output, &$ret);

$lBinaryDir = substr($lTarBall, 0, strlen($lTarBall)-strlen('.tar.gz'));

// create link
echo "ln -s $lProductDir . '/' . $lBinaryDir $lProductDir . '/' . 'mysql-' . $lVersion\n";
symlink($lProductDir . '/' . $lBinaryDir, $lProductDir . '/' . 'mysql-' . $lVersion);

exit($rc);

?>
