#!/usr/bin/env php
<?php

$rc = 0;

$lHomeDir = $_ENV['HOME'];
$lDebug   = $_ENV['DEBUG'];
$lMyName  = __FILE__;
$lBaseDir = dirname(dirname($lMyName));

if ( $lDebug != '' ) {
  echo "DEBUG: basedir=$lBaseDir\n";
}

$lConfFile         = $lBaseDir . '/' . 'etc/' . 'myenv.conf';
$lConfFileTemplate = $lConfFile . '.template';
$lConfFileBak      = $lConfFile . ".bak";

if ( $lDebug != '' ) {
  echo "DEBUG: ConfFile: $lConfFile\n";
}

// Check if myenv.conf exists
// Keep/Remove/Create

if ( ! file_exists($lConfFile) ) {
  // Copy template
  echo "Copy $lConfFileTemplate to $lConfFile\n";
  if ( ! copy($lConfFileTemplate, $lConfFile) ) {
    $rc = 1;
    echo "Cannot copy template $lConfFileTemplate to $lConfFile\n";
    exit($rc);
  }
}

$aConfiguration = parse_ini_file($lConfFile, true);

// extract the instances
$aInstance = array();
foreach ( $aConfiguration as $key => $value ) {
  if ( $key != 'default' ) {
    array_push($aInstance, $key);
  }
}

echo "\n";
echo "We configure the MySQL instances now. An instance is the same as a mysqld process.\n";

// do the default values first

// ...

if ( count($aInstance) == 0 ) {

  echo "\n";
  do {
    $input = strtolower(readline("You have no instance configured yet. Do you want to configure an instance now (Y/n)? "));
    if ( $input == '' ) {
      $input = 'y';
    }
  } while ( ($input != 'y') && ($input != 'n') );

  // Add a new instance name
  if ( $input == 'y' ) {

    echo "You have to add a new instance name. An instance name can consist of 0-9, a-z, A-Z, - and _.\n";
    echo "Examples: test, prod, dev, mysqld5513 mysqld-3306\n";
    echo "\n";
    $instance = readline("Enter your instance name: ");
    if ( $instance != '' ) {
      // todo: We have to do a check here!
      array_push($aInstance, $instance);
      $aConfiguration[$instance] = array();
    }
  }
}

// ask to add more instances...
do {
  do {
    echo "\n";
    $input = strtolower(readline("Do you want to add an other instance (N/y)? "));
    if ( $input == '' ) {
      $input = 'n';
    }
  } while ( ($input != 'n') && ($input != 'y') );

  if ( $input == 'y' ) {
    echo "\n";
    $instance = readline("Enter your instance name: ");
    if ( $instance != '' ) {
      // todo: We have to do a check here!
      array_push($aInstance, $instance);
      $aConfiguration[$instance] = array();
    }
  }
} while ( $input != 'n' );

// loop over all the instances
foreach ( $aInstance as $instance ) {

  echo "\n";
  echo "[$instance]\n";
  echo "\n";
  echo "basedir = " . $aConfiguration[$instance]['basedir'] . "\n";
  echo "datadir = " . $aConfiguration[$instance]['datadir'] . "\n";
  echo "port    = " . $aConfiguration[$instance]['port'] . "\n";
  echo "socket  = " . $aConfiguration[$instance]['socket'] . "\n";
  echo "my.cnf  = " . $aConfiguration[$instance]['my.cnf'] . "\n";
  echo "\n";

  do {
    $input = strtolower(readline("What do you want to do now. Delete, change or skip this instance (d/c/S)? "));
    if ( $input == '' ) {
      $input = 's';
    }
  } while ( ($input != 'd') && ($input != 'c') && ($input != 's') );

  switch ( $input ) {
  case 'd':
    unset($aConfiguration[$instance]);
    continue;
    break;
  case 'c':
    echo "\n";
    echo "Changing instance $instance\n";
    echo "\n";
    echo "Example: /usr/local/mysql or /var/lib/mysql or $lHomeDir/product/mysql-5.5.13";
    if ( $aConfiguration[$instance]['basedir'] != '' ) {
      echo " or [" . $aConfiguration[$instance]['basedir'] . "]";
    }
    echo "\n";
    $input = rtrim(readline("basedir = "), '/');
    if ( $input != '' ) {
      $aConfiguration[$instance]['basedir'] = $input;
    }
    if ( ! is_dir($aConfiguration[$instance]['basedir']) ) {
      echo "WARNING: directory " . $aConfiguration[$instance]['basedir'] . " is NOT a directory or does not exist.\n";
    }

    echo "\n";
    $default = $aConfiguration[$instance]['datadir'] != '' ? $aConfiguration[$instance]['datadir'] : "$lHomeDir/data/$instance";
    echo "Example: /usr/local/mysql/data or /var/lib/mysql/data or " . $aConfiguration[$instance]['basedir'] . "/data [$lHomeDir/data/$instance] or $default";
    echo "\n";
    $aConfiguration[$instance]['datadir'] = rtrim(readline("datadir = "), '/');
    if ( $aConfiguration[$instance]['datadir'] == '' ) {
      $aConfiguration[$instance]['datadir'] = $default;
    }
    if ( is_dir($aConfiguration[$instance]['datadir']) ) {
      echo "WARNING: directory " . $aConfiguration[$instance]['datadir'] ." already exist.\n";
    }
    else {
      echo "Creating directory " . $aConfiguration[$instance]['datadir'] . "\n";
      mkdir($aConfiguration[$instance]['datadir']);
    }
    echo "\n";

    $default = $aConfiguration[$instance]['port'] != '' ? $aConfiguration[$instance]['port'] : 3306;
    echo "Example: 3307 or [$default]\n";
    $input = intval(readline("port    = "));
    if ( $input == 0 ) {
      $aConfiguration[$instance]['port'] = $default;
    }
    else {
      $aConfiguration[$instance]['port'] = $input;
    }
    // Check if port already is in use
    echo "\n";

    $default = $aConfiguration[$instance]['socket'] == '' ? "/tmp/mysql-" . $aConfiguration[$instance]['port'] . ".sock" : $aConfiguration[$instance]['socket'];
    echo "Example: /tmp/mysql-" . $aConfiguration[$instance]['port'] . ".sock or [$default]\n";
    $aConfiguration[$instance]['socket']  = readline("socket  = ");
    if ( $aConfiguration[$instance]['socket'] == '' ) {
      $aConfiguration[$instance]['socket'] = $default;
    }
    echo "\n";

    $default = $aConfiguration[$instance]['my.cnf'] == '' ? $aConfiguration[$instance]['datadir'] . "/my.cnf" : $aConfiguration[$instance]['my.cnf'];
    echo "Example: /etc/my-" . $aConfiguration[$instance]['port'] . ".cnf or /etc/mysql/my-" . $aConfiguration[$instance]['port'] . ".cnf or " . $aConfiguration[$instance]['datadir'] . "/my-" . $aConfiguration[$instance]['port'] . ".cnf or [$default]\n";
    $aConfiguration[$instance]['my.cnf']  = readline("my.cnf  = ");
    if ( $aConfiguration[$instance]['my.cnf'] == '' ) {
      $aConfiguration[$instance]['my.cnf'] = $default;
    }
    if ( ! is_file($aConfiguration[$instance]['my.cnf']) ) {
      echo "WARNING: file " . $aConfiguration[$instance]['my.cnf'] ." is NOT readable or does not exist.\n";
    }
    echo "\n";

    break;
  case 's':
    continue;
    break;
  default:
    $rc = 2;
    echo "ERROR: #$rc ($input). Please report a bug!\n";
  }
}

// Writing the configuration file
echo "\n";
echo "Writing the configuration file...\n";

if ( file_exists($lConfFileBak) ) {
  unlink($lConfFileBak);
}
echo "Rename $lConfFile to $lConfFileBak\n";
if ( rename($lConfFile, $lConfFileBak) ) {

  echo "Write $lConfFile\n";
  if ( $fh = fopen($lConfFile, 'w') ) {

    foreach ( $aConfiguration as $instance => $record ) {
      fwrite($fh, "[$instance]\n\n");
      foreach ( $record as $key => $value ) {
        fwrite($fh, sprintf("%-20s = %s\n", $key, $value));
      }
      fwrite($fh, "\n\n");
    }
    fclose($fh);
  }
}
else {
  $rc = 3;
  echo "Renaming the configuration file $lConfFile to $lConfFileBak failed.\n";
  exit($rc);
}

// Adding myenv.profile to interactive shell startup file ~/.bash_profile
if ( is_readable("$lHomeDir/.bash_profile") ) {
  $tag = "# BEGIN MyEnv";
  $cmd = "grep --max-count=1 '$tag' $lHomeDir/.bash_profile";
  # echo "$cmd\n";
  $stdout = exec($cmd, &$output, &$ret);
}
else {
  $ret = 1;
}

if ( $ret == 1 ) {

  $lSourceSequence  = "export MYENV_BASE=$lHomeDir/myenv\n";
  $lSourceSequence .= 'MYENV_PWD=`pwd`' . "\n";
  $lSourceSequence .= 'cd $MYENV_BASE/bin' . "\n";
  $lSourceSequence .= ". myenv.profile\n";
  $lSourceSequence .= 'cd $MYENV_PWD' . "\n";

  echo "Do you want to add the following code to your ~/.bash_profile file?";
  echo "\n\n";
  echo $lSourceSequence;
  echo "\n";
  do {
    $input = strtolower(readline("(Y/n)? "));
    if ( $input == '' ) {
      $input = 'y';
    }
  } while ( ($input != 'n') && ($input != 'y') );

  if ( $input == 'y' ) {

    if ( $fh = fopen("$lHomeDir/.bash_profile", 'a') ) {

      fwrite($fh, "\n");
      fwrite($fh, "# BEGIN MyEnv\n");
      fwrite($fh, "# Written by the MyEnv phpInstaller.php script.\n");
      fwrite($fh, $lSourceSequence);
      fwrite($fh, "# END MyEnv\n");
      fclose($fh);
    }
  }
}

echo "\n";
echo "Now source your profile as follows:\n";
echo ". ~/.bash_profile\n";
echo "\n";
echo "The README gives some hints how to progress...\n";
echo "\n";

// Adding myenv_start_stop script to your start stop tree.
// echo "\n";
// echo "Now you should link the script $lBaseDir/bin/myenv.server into your OS startup process.\n";

exit($rc);

?>
