#!/usr/bin/env php
<?php

$rc = 0;

if ( count($_ENV) == 0 ) {

  $vo = ini_get('variables_order');
  echo "ERROR: variables_order MUST contain the E in /etc/php/cli/php.ini but seems not: variables_order = " . $vo . "\n";
  exit(2);
}

$lDebug    = isset($_ENV['DEBUG']) ? strval($_ENV['DEBUG']) : '';
$basedir   = strval($_ENV['MYENV_BASE']);
require("$basedir/lib/myEnv.inc");

$tmp           = strval($argv[1]);
$pDatabaseName = isset($argv[2]) ? strval($argv[2]) : '';

if ( $lDebug != '' ) {
  echo "DEBUG: tmp file: $tmp\n";
  echo "DEBUG: database: $pDatabaseName\n";
}

$file = '/etc/my.cnf';
if ( file_exists($file) ) {
  echo "Warning: $file exists. This can srew up myenv.\n";
}

$file = '/etc/mysql/my.cnf';
if ( file_exists($file) ) {
  echo "Warning: $file exists. This can srew up myenv.\n";
}

$file = '/usr/local/mysql/etc/my.cnf';
if ( file_exists($file) ) {
  echo "Warning: $file exists. This can srew up myenv.\n";
}

$file = "~/.my.cnf";
if ( file_exists($file) ) {
  echo "Warning: $file exists. This can srew up myenv.\n";
}

$lConfigurationFile = $basedir . '/etc/myenv.conf';
if ( ! is_file($lConfigurationFile) ) {
  echo "Warning: Configuration file $lConfigurationFile does not exist!\n";
  echo "         Please create or copy from $lConfigurationFile.template\n";
  exit(1);
}

$aConfigurationFile = parse_ini_file($lConfigurationFile, true);
$aDbNames = array();
foreach ( $aConfigurationFile as $dbname => $paramter ) {
  if ( $dbname != 'default' ) {
    array_push($aDbNames, $dbname);
  }
}

$lDbName = '';
if ( $pDatabaseName != '' ) {
  $lDbName = $pDatabaseName;
}
elseif ( isset($_ENV['MYENV_DATABASE']) ) {
  $lDbName = $_ENV['MYENV_DATABASE'];
}
else {
  $lDbName = $aDbNames[0];
}

if ( $lDebug != '' ) {
  echo "DEBUG: database: $lDbName\n";
}

$_ENV['INFODIR']         = isset($_ENV['INFODIR']) ? $_ENV['INFODIR'] : '';
$_ENV['INFOPATH']        = isset($_ENV['INFOPATH']) ? $_ENV['INFOPATH'] : '';
# $_ENV['MANPATH']         = isset($_ENV['MANPATH']) ? $_ENV['MANPATH'] : '';
$_ENV['LD_LIBRARY_PATH'] = isset($_ENV['LD_LIBRARY_PATH']) ? $_ENV['LD_LIBRARY_PATH'] : '';

// getEnv
$aDatabaseParameter = $aConfigurationFile[$lDbName];

$file = $aDatabaseParameter['basedir'] . '/my.cnf';
if ( file_exists($file) ) {
  echo "Warning: $file exists. This can srew up myenv.\n";
}

// to avoid a complete mess:
$path = $_ENV['PATH'];
if ( $aDatabaseParameter['basedir'] != '' ) {
  if ( is_dir($aDatabaseParameter['basedir'] . '/bin') ) {
    $path = addDirectoryToPath($path, $aDatabaseParameter['basedir'] . '/bin', '');
  }
  if ( is_dir($aDatabaseParameter['basedir'] . '/libexec') ) {
    $path = addDirectoryToPath($path, $aDatabaseParameter['basedir'] . '/libexec', '');
  }
  if ( is_dir($aDatabaseParameter['basedir'] . '/scripts') ) {
    $path = addDirectoryToPath($path, $aDatabaseParameter['basedir'] . '/scripts', '');
  }
}

if ( is_dir($basedir . '/bin') ) {
  $path = addDirectoryToPath($path, $basedir . '/bin', '');
}
if ( is_dir($basedir . '/libexec') ) {
  $path = addDirectoryToPath($path, $basedir . '/libexec', '');
}
if ( is_dir($basedir . '/scripts') ) {
  $path = addDirectoryToPath($path, $basedir . '/scripts', '');
}

$fh = fopen($tmp, 'w');
fwrite($fh, "export MYSQL_HOME=" . $aDatabaseParameter['basedir'] . "\n");
fwrite($fh, "export PATH=$path\n");
fwrite($fh, "export INFODIR=" . addDirectoryToPath($_ENV['INFODIR'], $aDatabaseParameter['basedir'] . "/docs", $aDatabaseParameter['basedir'] . "/docs") . "\n");
fwrite($fh, "export INFOPATH=" . addDirectoryToPath($_ENV['INFOPATH'], $aDatabaseParameter['basedir'] . "/docs", $aDatabaseParameter['basedir'] . "/docs") . "\n");
# fwrite($fh, "export MANPATH=" . addDirectoryToPath($_ENV['MANPATH'], $aDatabaseParameter['basedir'] . "/man", $aDatabaseParameter['basedir'] . "/man") . "\n");
fwrite($fh, "export LD_LIBRARY_PATH=" . addDirectoryToPath($_ENV['LD_LIBRARY_PATH'], $aDatabaseParameter['basedir'] . "/lib", $aDatabaseParameter['basedir'] . "/lib") . "\n");
fwrite($fh, "export MYSQL_TCP_PORT=" . $aDatabaseParameter['port'] . "\n");
fwrite($fh, "export MYSQL_UNIX_PORT=" . $aDatabaseParameter['socket'] . "\n");
fwrite($fh, "export MYENV_DATABASE=" . $lDbName . "\n");
fwrite($fh, "export MYENV_DATADIR=" . $aDatabaseParameter['datadir'] . "\n");
fwrite($fh, 'export MYSQL_PS1=\'\u@\h ' . $lDbName . ' [\d]> \'' . "\n");
foreach ( $aDbNames as $dbname ) {
  fwrite($fh, "alias $dbname='setMyEnv $dbname'\n");
}
fwrite($fh, "alias cdh='cd " . $aDatabaseParameter['basedir'] . "'\n");
fwrite($fh, "alias cdd='cd " . $aDatabaseParameter['datadir'] . "'\n");
fwrite($fh, "alias cde='cd " . $basedir . "'\n");
fwrite($fh, "alias cdl='cd " . $basedir . '/log' . "'\n");

// Only if really exists!
if ( isset($aDatabaseParameter['clusterdir']) ) {
  if ( is_dir($aDatabaseParameter['clusterdir']) ) {
    fwrite($fh, "alias cdc='cd " . $aDatabaseParameter['clusterdir'] . "'\n");
  }
  else {
  fwrite($fh, "alias cdc='echo directory " . $aDatabaseParameter['clusterdir'] . "does not exist.'\n");
  }
}
else {
  fwrite($fh, "alias cdc='echo no clusterdir defined'\n");
}

fwrite($fh, "alias ll='ls -l'\n");
fwrite($fh, "alias la='ls -la'\n");

fclose($fh);

exit($rc);

?>
