#!/usr/bin/env php
<?php

$rc = 0;

$lHome     = $_ENV['HOME'] . '/product';
$lDebug    = isset($_ENV['DEBUG']) ? $_ENV['DEBUG'] : 0;
$basedir   = $_ENV['MYENV_BASE'];

$lConfigurationFile = $basedir . '/etc/myenv.conf';
if ( ! is_file($lConfigurationFile) ) {
  echo "Warning: Configuration file $lConfigurationFile does not exist!\n";
  echo "         Please create or copy from $lConfigurationFile.template\n";
  exit(1);
}

require_once("$basedir/lib/myEnv.inc");

$aConfiguration = parse_ini_file($lConfigurationFile, true);
$aDbNames = getSectionTitles($aConfiguration);

// Get available release versions

$aReleaseVersion = array();
foreach ( glob("$lHome/*", GLOB_ONLYDIR) as $dir ) {

  if ( preg_match("/mysql-(\d{1,2}\.\d{1,2}\.\d{1,2})/", $dir, &$matches) ) {
    array_push($aReleaseVersion, $matches[1]);
  }
}
$aReleaseVersion = array_unique($aReleaseVersion, SORT_REGULAR);

if ( $lDebug != '' ) {
  var_dump($aReleaseVersion);
}

// Get version of each instance
foreach ( $aConfiguration as $db => $value ) {

  if ( $db != 'default' ) {
    if ( preg_match("/(\d{1,2}\.\d{1,2}\.\d{1,2})/", $aConfiguration[$db]['basedir'], &$matches) ) {
      $aConfiguration[$db]['version'] = $matches[1];
    }
    else {
      $aConfiguration[$db]['version'] = 'unknown';
    }
  }
}

echo "\n";
echo "Releases : " . implode(' ', $aReleaseVersion) . "\n";
echo "\n";

// Display up and down instances

$aUp   = array();
$aDown = array();
foreach ( $aDbNames as $db ) {

  if ( count($aConfiguration[$db]) == 0 ) {
    $rc = 7;
    echo "ERROR #$rc\n";
    continue;
  }

  // todo: use from library!!!
  $cmd = $aConfiguration[$db]['basedir'] . "/bin/mysqladmin --socket=" . $aConfiguration[$db]['socket'] . " ping 1>/dev/null 2>&1";
  debug($cmd);
  system($cmd, &$ret);

  if ( $ret != 0 ) {
    $aDown[$db] = "$db (" . $aConfiguration[$db]['version'] . ")";
  }
  else {
    $aUp[$db]   = "$db (" . $aConfiguration[$db]['version'] . ")";
  }
}

if ( $lDebug != '' ) {
  var_dump($aUp);
  var_dump($aDown);
}

echo "Up       : " . implode(' ', $aUp) . "\n";
echo "\n";
echo "Down     : " . implode(' ', $aDown) . "\n";
echo "\n";

foreach ( $aDbNames as $db ) {

  $aSchema = getSchemaNames($aConfiguration[$db]['datadir']);
  printf("%-16s (%5d) : %s\n", $db, $aConfiguration[$db]['port'], implode(' ', $aSchema));
}
echo "\n";

exit($rc);

?>
